//mrtu.c
#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <unistd.h>
#include <fcntl.h>
#include <termios.h>
#include "mrtu.h"
#include "types.h"

int	fd;
struct termios serial;
U8	rxbuf[_szbuf];
U8	txbuf[_szbuf];

inline U16 MRTU_CRC(BYTE* data, ushort len)
{
        ushort res_CRC = 0xFFFF;
        ushort count = 0;
        BYTE count_crc;
        BYTE dt;
        while(count < len)
        {
                //
                count_crc = 0;
                dt = (BYTE)(data[count]);
                res_CRC ^= (ushort)(dt);
                //
                while(count_crc < 8)
                {
                        if((res_CRC & 0x0001) < 1)
                        {
                                res_CRC = (res_CRC >> 1) & 0x7FFF;
                        }
                        else
                        {
                                res_CRC = (res_CRC >> 1) & 0x7FFF;
                                res_CRC ^= 0xA001;
                        };
                        count_crc++;
                };
                count++;
        }
        return (res_CRC);
}


int MRTU_Connect(char* strPort)
{
	fd = open(strPort, O_RDWR | O_NOCTTY | O_NDELAY);
	//
	if (fd == -1) {
        	perror(strPort);
        	return -1;
    	}
	//
	if (tcgetattr(fd, &serial) < 0) {
        	perror("Getting configuration");
        	return -1;
    	}
	// Set up Serial Configuration
/*    	serial.c_iflag = 0;
    	serial.c_oflag = 0;
    	serial.c_lflag = 0;
    	serial.c_cflag = 0;
 
    	serial.c_cc[VMIN] = 0;
    	serial.c_cc[VTIME] = 0;
*/ 
    	serial.c_cflag = B9600| CS8 | CREAD;
 
    	tcsetattr(fd, TCSANOW, &serial); // Apply configuration
	//
	return 0;
}

int MRTU_Disconnect(void)
{
	close(fd);
	return 0;
}

//команда 6 - запись в один регистр
int MRTU_CMD_6(U8 Dev, U16 Addr, U16 Reg)
{
	U16		crc;
	int 		wcount;
	int 		rcount;
	//printf("Reg %u\n", Reg);
	//
	txbuf[0]	= Dev;	//адрес
	txbuf[1]	= 6;	//команда
	txbuf[2]	= (U8)((Addr & 0xFF00) >> 8);
	txbuf[3]	= (U8)(Addr & 0xFF);
	txbuf[4]	= (U8)((Reg & 0xFF00) >> 8);
	txbuf[5]	= (U8)(Reg & 0xFF);
	//
	crc 		= MRTU_CRC(&txbuf[0], 6);
	//txbuf[7]	= (U8)((crc >> 8) & 0xFF);
	//txbuf[6]	= (U8)(crc & 0xFF);
	(*(U16*)(&txbuf[6]))	= crc;
	//
	//printf("buf out: %.2X %.2X %.2X %.2X %.2X %.2X %.2X %.2X\n",txbuf[0],txbuf[1],txbuf[2],txbuf[3],txbuf[4],txbuf[5],txbuf[6],txbuf[7]);
	//
	wcount = write(fd, &txbuf[0], 8);
    	if (wcount < 0) {
        	perror("Write");
        	return -1;
    	}
	//printf("sended: %i\n", wcount);
	//
	usleep(75000);
	memset(&rxbuf[0], 0, _szbuf);
	//fcntl( fd, F_SETFL, 0);
	rcount = read(fd, &rxbuf[0], _szbuf);
    	if (rcount < 0) {
        	//perror("Read");
        	return -1;
    	}
    	else {
        	//printf("Received %d characters\n", rcount);
		//printf("=== %s\n", &rxbuf[0]);
		//printf("buf in: %.2X %.2X %.2X %.2X %.2X %.2X %.2X %.2X\n",rxbuf[0],rxbuf[1],rxbuf[2],rxbuf[3],rxbuf[4],rxbuf[5],rxbuf[6],rxbuf[7]);
    	}
	return 0;
}

//команда 16 - запись в несколько регистров
int MRTU_CMD_16(U8 Dev, U16 Addr, U16 * Reg, U16 CntReg)
{
	int 		wcount;
	int 		rcount;
	U16		crc;
	U8		cntbytes 	= (U8)(CntReg * 2);
	U8		addr 		= 0;
	U16		regs		= 0;
	txbuf[0]	= Dev;	//адрес
	txbuf[1]	= 16;	//команда
	txbuf[2]	= (U8)((Addr & 0xFF00) >> 8);
	txbuf[3]	= (U8)(Addr & 0xFF);
	txbuf[4]	= (U8)((CntReg & 0xFF00) >> 8);
	txbuf[5]	= (U8)(CntReg & 0xFF);
	txbuf[6]	= cntbytes;
	//
	while(addr < cntbytes)
	{
		txbuf[addr + 7] = (U8)(((*(U16*)(Reg + regs)) & 0xFF00)>> 8);
		addr++;
		txbuf[addr + 7] = (U8)((*(U16*)(Reg + regs)) & 0x00FF);
		addr++;
		regs++;
	}	
	//
	crc				= MRTU_CRC(&txbuf[0], cntbytes + 7);
	(*(U16*)(&txbuf[addr + 7]))	= crc;
	//
	//printf("buf out: %.2X %.2X %.2X %.2X %.2X %.2X %.2X %.2X\n",txbuf[0],txbuf[1],txbuf[2],txbuf[3],txbuf[4],txbuf[5],txbuf[6],txbuf[7]);
	wcount = write(fd, &txbuf[0], addr + 9);
	usleep(75000);
	memset(&rxbuf[0], 0, _szbuf);
	//fcntl( fd, F_SETFL, 0);
	rcount = read(fd, &rxbuf[0], _szbuf);
	if (rcount < 0) {
        	//perror("Read");
        	return -1;
    	}
    	else {
        	//printf("Received %d characters\n", rcount);
		//printf("=== %s\n", &rxbuf[0]);
		//printf("buf in: %.2X %.2X %.2X %.2X %.2X %.2X %.2X %.2X\n",rxbuf[0],rxbuf[1],rxbuf[2],rxbuf[3],rxbuf[4],rxbuf[5],rxbuf[6],rxbuf[7]);
    	}
	return 0;
}

